@@ -132,12 +132,19 @@ This single value is the total count of all cui_1, cui_2 bigram pairs.
 =head1 AUTHOR
 
  Keith Herbert, Virginia Commonwealth University
+ Amy Olex, Virginia Commonwealth University
 
 =head1 COPYRIGHT
 
 Copyright (c) 2015,
 Keith Herbert, Virginia Commonwealth University
-herbertkb at vcu edu
+herbertkb at vcu edu 
+
+Amy Olex, Virginia Commonwealth University
+alolex at vcu dot edu
+
+Bridget McInnes, Virginia Commonwealth University
+btmcinnes at vcu dot edu
 
 
 This program is free software; you can redistribute it and/or modify it under
@@ -164,6 +171,7 @@ this program; if not, write to:
 
 use strict;
 use warnings;
+use Time::HiRes;
 
 use Compress::Zlib;     # Read compress .gz files
 use Data::Dumper;       # Helps with debugging
@@ -283,18 +291,28 @@ for my $file (@files) {
 #                                   Main 
 ###############################################################################
 {
+	
 say "Connecting to $database on $hostname" if $VERBOSE;
 my $dbh = open_mysql_database($database, $hostname, $port, $username, $password);
 
+my($filetime_s,$filetime_e,$dbtime_s,$dbtime_e,$total_time_s);
+
+
+$total_time_s = Time::HiRes::gettimeofday() if $DEBUG;
+
 
 while (@files) {
     my @curr_files = splice @files, 0, $file_step;
-     
+    
+    $filetime_s = Time::HiRes::gettimeofday() if $DEBUG;
     my($bigram_ref, $n1p_ref, $np1_ref, $npp) = process_files(@curr_files);
-    
-    say "Entering scores into $database..." if $VERBOSE;    
+    $filetime_e = Time::HiRes::gettimeofday() if $DEBUG;
+	
+    say "Entering scores into $database..." if $VERBOSE;  
+	$dbtime_s = Time::HiRes::gettimeofday() if $DEBUG;
     update_database($dbh, $bigram_ref, $n1p_ref, $np1_ref, $npp);
-    
+    $dbtime_e = Time::HiRes::gettimeofday() if $DEBUG;
+	
     if ($DEBUG) {
         print Dumper($bigram_ref);
         print Dumper($n1p_ref);
@@ -303,6 +321,12 @@ while (@files) {
     }
 }
 
+my $total_time_e = Time::HiRes::gettimeofday() if $DEBUG;
+
+printf("Serial elapsed time for file processing only: %.2f sec\n", $filetime_e - $filetime_s) if $DEBUG;
+printf("Serial elapsed time for database update only: %.2f sec\n", $dbtime_e - $dbtime_s) if $DEBUG;
+printf("Serial elapsed time for entire Main: %.2f sec\n", $total_time_e - $total_time_s) if $DEBUG;
+
 # Close connection to database                       
 $dbh->disconnect;
 
@@ -603,10 +627,10 @@ sub count_bigrams {
     my @phrases = @$phrases_ref;
     
     # Iterate through n-1 phrases in utterance
-    for (my $i = 0; $i < $#phrases; $i++) {
+    for (my $i = 0; $i <= $#phrases; $i++) {
                 
         my @phrase_1 = @{ $phrases[$i]      };  # Mappings for current phrase
-        my @phrase_2 = @{ $phrases[$i + 1]  };  # Mappings for the next phrase
+        my @phrase_2 = @{ $phrases[$i + 1]  } if $i < $#phrases;;  # Mappings for the next phrase
 
         my %prior;  # Tracks bigrams within same phrase to avoid double counting
         
@@ -621,13 +645,14 @@ sub count_bigrams {
                 
                 $incrementor->($cui_1, $cui_2, \%prior);
             }
-         
-            # Count the kth CUI with the first of each of the next phrases maps
-            foreach my $map_str_p2 ( @phrase_2 ) {
-                (my $first_of_next_phrase) = $map_str_p2 =~ /(C\d{7})/;
+         	if ( $i < $#phrases ) {
+            	# Count the kth CUI with the first of each of the next phrases maps
+            	foreach my $map_str_p2 ( @phrase_2 ) {
+                	(my $first_of_next_phrase) = $map_str_p2 =~ /(C\d{7})/;
                 
-                $incrementor->($cuis[-1], $first_of_next_phrase, \%prior);
-            }
+               	 	$incrementor->($cuis[-1], $first_of_next_phrase, \%prior);
+            	}
+			}
         }
     }
 }
